package wood.keith.opentools.jbimages;

import java.io.File;
import java.io.IOException;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;

import com.borland.jbuilder.JBuilderMenu;
import com.borland.primetime.PrimeTime;
import com.borland.primetime.actions.UpdateAction;
import com.borland.primetime.util.Classes;
import com.borland.primetime.util.ZipIndex;
import com.borland.primetime.util.ZipIndexEntry;

/**
 * Display the large images JBuilder uses for its wizards.
 *
 * @author   Keith Wood (kbwood@iprimus.com.au)
 * @version  1.0  3 May 2002
 * @version  1.1  21 July 2003
 */
public class JBImages extends JFrame {

  private static final int MIN_SIZE = 10000;

  private JComboBox _imageCombo = new JComboBox();
  private JLabel _imageLabel = new JLabel();

  /**
   * Start up the JBImages tool from the Help menu.
   *
   * @param  majorVersion  the major version of the current OpenTools API
   * @param  minorVersion  the minor version of the current OpenTools API
   */
  public static void initOpenTool(byte majorVersion, byte minorVersion) {
    if (majorVersion != PrimeTime.CURRENT_MAJOR_VERSION) {
      return;
    }
    JBuilderMenu.GROUP_Help2.add(ACTION_JBImages);
    if (PrimeTime.isVerbose()) {
      System.out.println("Loaded JBImages");
      System.out.println("Written by Keith Wood (kbwood@iprimus.com.au)");
    }
  }

  /**
   * The action that invokes the image display.
   */
  public static UpdateAction ACTION_JBImages = new UpdateAction(
    "JBuilder images...", 'I', "Show JBuilder's large wizard images") {

      /**
       * Bring up the frame with the images.
       */
      public void actionPerformed(ActionEvent event) {
        JBImages jbImages = new JBImages();
        jbImages.pack();
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        jbImages.setLocation((int)(screen.getWidth() - jbImages.getWidth()) / 2,
          (int)(screen.getHeight() - jbImages.getHeight()) / 2);
        jbImages.show();
      }
    };

  public JBImages() {
    try {
      jbInit();
      loadImages();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Initialise the UI.
   */
  private void jbInit() throws Exception {
    _imageLabel.setMinimumSize(new Dimension(170, 320));
    _imageLabel.setPreferredSize(new Dimension(170, 320));
    _imageLabel.setHorizontalAlignment(SwingConstants.CENTER);
    this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    this.setTitle("JBuilder Wizard Images");
    this.getContentPane().add(_imageLabel, BorderLayout.CENTER);
    this.getContentPane().add(_imageCombo, BorderLayout.SOUTH);
  }

  /**
   * Find the images over 20000 bytes in the jbuilder.jar file
   * and add their names to the combo box.
   */
  private void loadImages() {
    String prJar =
      Classes.getRootEntryFromClasspath("com.borland.primetime.PrimeTime");
    loadImages(ZipIndex.getZipIndex(new File(prJar)));
    String jbJar =
      Classes.getRootEntryFromClasspath("com.borland.jbuilder.JBuilder");
    if (!jbJar.equals(prJar)) {
      loadImages(ZipIndex.getZipIndex(new File(jbJar)));
    }
    // Defer image display until after loading
    _imageCombo.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        _imageCombo_actionPerformed(e);
      }
    });
    _imageCombo.setSelectedIndex(0);
  }

  /**
   * Find the images over 20000 bytes in the jbuilder.jar file
   * and add their names to the combo box.
   */
  private void loadImages(ZipIndex jar) {
    ZipIndexEntry[] entries = jar.getZipIndexEntries();
    for (int index = 0; index < entries.length; index++) {
      if (entries[index].getName().endsWith(".gif")) {
        // Find images
        try {
          // Find their size
          byte[] contents = jar.read(entries[index]);
          if (contents.length > MIN_SIZE) {
            // And add them
            _imageCombo.addItem("/" +
              entries[index].getDirectory().replace('\\', '/') + "/" +
              entries[index].getName());
          }
        }
        catch (IOException ex) {
          // Ignore
        }
      }
    }
  }

  /**
   * Display the image for the selected file.
   */
  private void _imageCombo_actionPerformed(ActionEvent e) {
    String imageName = (String)_imageCombo.getSelectedItem();
    _imageLabel.setIcon(new ImageIcon(this.getClass().getResource(imageName)));
  }
}